# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('howl', '0003_alert_value'),
    ]

    operations = [
        migrations.AlterField(
            model_name='observer',
            name='operator',
            field=models.CharField(max_length=32, verbose_name='Operator type', choices=[('EqualOperator', 'Equals'), ('GreaterThanOperator', 'Greater than'), ('LowerThanOperator', 'Lower than')]),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DjangoHowlConfig(AppConfig):
    name = "howl"
    verbose_name = _("Howl")

    def ready(self):
        from howl.models import do_operator_setup

        model = self.get_model("Observer")
        do_operator_setup(model)



# Generated by Django 2.1.4 on 2018-12-08 03:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('howl', '0008_auto_20170101_0830'),
    ]

    operations = [
        migrations.AlterField(
            model_name='observer',
            name='operator',
            field=models.CharField(choices=[('EqualOperator', 'Equals'), ('GreaterThanOperator', 'Greater than'), ('LowerThanOperator', 'Lower than')], max_length=32, verbose_name='Operator type'),
        ),
    ]



from django.utils.translation import gettext_lazy as _


class OperatorException(Exception):
    pass


class BaseOperator(object):
    display_name = None

    def __init__(self, observer):
        self.observer = observer

    def compare(self, compare_value):
        raise NotImplementedError("Method compare is not implemented.")


class EqualOperator(BaseOperator):
    display_name = _("Equals")

    def compare(self, compare_value):
        return self.observer.value == compare_value


class LowerThanOperator(BaseOperator):
    display_name = _("Lower than")

    def compare(self, compare_value):
        return compare_value < self.observer.value


class GreaterThanOperator(BaseOperator):
    display_name = _("Greater than")

    def compare(self, compare_value):
        return compare_value > self.observer.value



from datetime import timedelta
from functools import partial

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils import timezone
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from .conf.settings import HOWL_OPERATORS
from .operators import BaseOperator
from .signals import alert_clear, alert_notify, alert_wait


def do_operator_setup(cls, **kwargs):
    operators = {}

    for operator_path in getattr(settings, "HOWL_OPERATORS", HOWL_OPERATORS):
        operator_class = import_string(operator_path)

        if not issubclass(operator_class, BaseOperator):
            raise TypeError(
                'Operator "{0}" must be of type: "howl.operators.BaseOperator"'.format(
                    operator_class
                )
            )

        operator_name = operator_class.__name__

        if operator_name in operators:
            raise ImproperlyConfigured(
                'Operator named "{0}" already exists.'.format(operator_name)
            )

        operators[operator_name] = operator_class

    choices = [
        (name, operator_class.display_name)
        for name, operator_class in operators.items()
    ]

    cls.operators = operators
    cls.operator_choices = choices

    operator = cls._meta.get_field("operator")
    operator.choices.extend(cls.operator_choices)
    operator.choices.sort(key=lambda item: item[0])

    cls.get_operator_display = partial(cls._get_FIELD_display, field=operator)


class Observer(models.Model):
    name = models.CharField(_("Name"), max_length=255)
    operator = models.CharField(_("Operator type"), max_length=32, choices=[])
    value = models.FloatField(_("Value"))
    waiting_period = models.PositiveIntegerField(
        _("Waiting period"), help_text=_("In seconds")
    )
    alert_every_time = models.BooleanField(_("Alert every time"), default=False)

    class Meta:
        verbose_name = _("Observer")
        verbose_name_plural = _("Observers")
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_alert_identifier(self, **kwargs):
        if kwargs.get("identifier", None):
            return kwargs["identifier"]

        return "howl-observer:{0}".format(self.pk)

    def get_alert(self, compare_value, **kwargs):
        operator_class = self.operators[self.operator]

        if operator_class(self).compare(compare_value):
            Alert.clear(compare_value, observer=self, **kwargs)
            return None

        return Alert.set(compare_value, observer=self, **kwargs)

    def compare(self, compare_value, **kwargs):
        return self.get_alert(compare_value, **kwargs) is None


class Alert(models.Model):
    STATE_WAITING, STATE_NOTIFIED = range(0, 2)
    STATE_CHOICES = (
        (STATE_WAITING, _("Waiting")),
        (STATE_NOTIFIED, _("Notified")),
    )
    identifier = models.CharField(_("Identifier"), max_length=64, unique=True)
    timestamp = models.DateTimeField(auto_now_add=True)
    value = models.CharField(_("Value"), max_length=255, blank=True, null=True)
    state = models.PositiveSmallIntegerField(
        _("State"), choices=STATE_CHOICES, default=STATE_WAITING
    )

    class Meta:
        verbose_name = _("Alert")
        verbose_name_plural = _("Alerts")
        ordering = ("-timestamp",)

    def __str__(self):
        return _("Alert for {0}").format(self.identifier)

    @classmethod
    def set(cls, value=None, **kwargs):
        if "observer" in kwargs:
            identifier = kwargs["observer"].get_alert_identifier(**kwargs)
            waiting_period = kwargs["observer"].waiting_period
            alert_every_time = kwargs["observer"].alert_every_time
        else:
            if "identifier" not in kwargs:
                raise ValueError("`observer` or `identifier` required.")
            identifier = kwargs["identifier"]
            waiting_period = kwargs.get("waiting_period", 0)
            alert_every_time = kwargs.get("alert_every_time", False)

        obj, created = Alert.objects.get_or_create(
            identifier=identifier, defaults={"value": value}
        )

        if created and waiting_period > 0:
            alert_wait.send(sender=cls, instance=obj, value=value, **kwargs)
            return obj

        alert_time = obj.timestamp + timedelta(seconds=waiting_period)

        if alert_time < timezone.now():
            if obj.state == obj.STATE_NOTIFIED and alert_every_time:
                obj.value = value
                obj.save(update_fields=["value"])
                alert_notify.send(sender=cls, instance=obj, value=value, **kwargs)

            if obj.state == obj.STATE_WAITING:
                obj.value = value
                obj.state = obj.STATE_NOTIFIED
                obj.save(update_fields=["value", "state"])
                alert_notify.send(sender=cls, instance=obj, value=value, **kwargs)

        return obj

    @classmethod
    def clear(cls, value=None, **kwargs):
        if "observer" in kwargs:
            identifier = kwargs["observer"].get_alert_identifier(**kwargs)
        else:
            if "identifier" not in kwargs:
                raise ValueError("`observer` or `identifier` required.")
            identifier = kwargs["identifier"]

        try:
            obj = Alert.objects.get(identifier=identifier)
            alert_clear.send(sender=cls, instance=obj, value=value, **kwargs)
            obj.delete()
        except Alert.DoesNotExist:
            pass



import os
from codecs import open

from setuptools import find_packages, setup


BASE_DIR = os.path.abspath(os.path.dirname(__file__))
VERSION = __import__("howl").__version__


with open(os.path.join(BASE_DIR, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="django-howl",
    version=VERSION,
    description=(
        "Django app to observe almost everything you want and "
        "pushes notifications to signals."
    ),
    long_description=long_description,
    url="https://github.com/deluge/django-howl",
    project_urls={
        "Bug Reports": "https://github.com/deluge/django-howl/issues",
        "Source": "https://github.com/deluge/django-howl",
    },
    author="Benjamin Banduhn, Stephan Jaekel",
    author_email="deluge@banduhn.com",
    packages=find_packages(exclude=["examples", "docs", "tests", "tests.*"]),
    install_requires=["Django>=2.2,<3.3"],
    extras_require={
        "dev": [
            "coverage[toml]>=5.3",
            "factory-boy>=3.1.0",
            "freezegun>=1.0.0",
            "pytest>=6.1.1",
            "pytest-black>=0.3.12",
            "pytest-cov>=2.10.1",
            "pytest-django>=3.10.0",
            "pytest-flake8>=1.0.6",
            "pytest-isort>=1.2.0",
            "sphinx>=3.2.1",
            "sphinx-rtd-theme==0.5.0",
            "vcrpy>=4.1.1",
        ],
    },
    include_package_data=True,
    keywords="django howl observer watchdog alert signal",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



from django.contrib import admin

from .models import Alert, Observer


@admin.register(Observer)
class ObserverAdmin(admin.ModelAdmin):
    list_display = ("name", "operator", "value")


@admin.register(Alert)
class AlertAdmin(admin.ModelAdmin):
    list_display = ("identifier", "timestamp", "value", "state")
    list_filter = ("state",)
